/* 
   nmidi - ALSA/MWPP MIDI client

   Copyright (C) 2003 Hans Fugal

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  

#include <termios.h>
#include <grp.h>
#include <pwd.h>
*/

#define MAX_PACKET_LEN 1024 /* maximum MWPP packet length */
#define MAX_EVENTS MAX_PACKET_LEN /* way too big, probably */
#define DEFAULT_PORT 4242
#define SRATE 128 /* sampling rate (events per second) */

void *outbound_thread(void*);
void *inbound_thread(void*);

/* buf shall be at least MAX_PACKET_LEN size.
 * returns length of packet encoded, or a negative number if not all events
 * were encoded */
int mwpp_encode(snd_seq_event_t *events, int count, unsigned char *buf);
/* decodes at most len bytes of buf into at most count events. returns the
 * number of events created. */
int mwpp_decode(unsigned char *buf, int len, snd_seq_event_t *events, 
		int count);

snd_seq_t *open_seq();
int portid;
snd_seq_t *seq_handle;

static void usage (int status);

void print_chars(char *head, unsigned char *buff, int length);

/* The name the program was run with, stripped of any leading path. */
char *program_name;

/* Option flags and variables */
int client;
int port;
int src_client, src_port;
int dest_client, dest_port;
int use_udp;
char *hostname;
unsigned long basetime;
static int decode_switches (int argc, char **argv);
static int parse_addr(char *arg, int *clientp, int *portp);
/*
static void reset_all(midi_status_t *st);
*/

typedef struct {
	unsigned v : 2;
	unsigned p : 1;
	unsigned x : 1;
	unsigned cc: 4;
	unsigned m : 1;
	unsigned pt: 7;
	unsigned seqnum : 16; /* end first 32-bit word */
	unsigned timestamp;
	unsigned ssrc;
} rtp_hdr;

typedef struct {
	unsigned b : 1;
	unsigned z : 1;
	unsigned p : 1;
	unsigned len : 5;
} mwpp_midi_command_hdr;
